<?php

namespace Drupal\idealyzer\Service;

use Drupal\Component\Serialization\Json;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

class SiteScannerService {
  protected $httpClient;
  protected $configFactory;

  public function __construct(Client $httpClient, $configFactory) {
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
  }

  /**
   * Fetches data from the Site Scanning API.
   *
   * @param string $url The URL of the site to scan.
   * @return array The response data or error message.
   */
  public function fetchData($url, $page = 1, $limit = 10) {
    $apiKey = $this->getApiKey();
    $apiUrl = 'https://api.gsa.gov/technology/site-scanning/v1/websites';

    try {
        $response = $this->httpClient->request('GET', $apiUrl, [
            'query' => [
                'final_url_domain' => $url,
                'api_key' => $apiKey,
                'page' => $page,
                'limit' => $limit
            ]
        ]);

        $statusCode = $response->getStatusCode();
        $contents = $response->getBody()->getContents();

        // Log the request and response for debugging
        \Drupal::logger('idealyzer')->info('Requested URL: @url with response status: @status', ['@url' => $url, '@status' => $statusCode]);

        if ($statusCode === 200) {
            return [
                'success' => true,
                'data' => Json::decode($contents),
                'links' => $response->getHeader('Link')
            ];
        } else {
            \Drupal::logger('idealyzer')->error('API error with status @status: @response', ['@status' => $statusCode, '@response' => $contents]);
            return [
                'success' => false,
                'error' => t('API responded with status code: @status', ['@status' => $statusCode])
            ];
        }
    } catch (GuzzleException $e) {
        watchdog_exception('idealyzer', $e);
        return [
            'success' => false,
            'error' => $e->getMessage()
        ];
    }
}



  /**
 * Retrieves the stored API key from configuration.
 *
 * @return string The API key.
 */
protected function getApiKey() {
    // Retrieve the actual API key from the configuration.
    return $this->configFactory->get('idealyzer.settings')->get('api_key');
}

  
}
